package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.GetActiveRes;
import com.abi.qms.platform.dto.res.ListActivePackage;
import com.abi.qms.platform.dto.res.ListActiveRes;
import com.abi.qms.platform.service.ActiveService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 活动管理
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@RequestMapping("active")
@Api(tags = "活动管理")
public class ActiveController {

    @Autowired
    private ActiveService activeService;


    @ApiOperation("新增活动")
    @PostMapping("addActive")
    public BaseResponse addActive(@Validated @RequestBody AddActiveReq addActiveReq) throws Exception{
        //新增活动
        activeService.addActive(addActiveReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("编辑活动")
    @PostMapping("updateActive")
    public BaseResponse updateActive(@Validated @RequestBody UpdateActiveReq updateActiveReq) throws Exception{
        //编辑活动
        activeService.updateActive(updateActiveReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("分页查询活动")
    @GetMapping("listActive")
    public BaseResponse<ListActiveRes> listActive(@Validated ListActiveReq listActiveReq) throws Exception{
        //分页查询活动
        ListActiveRes result = activeService.listActive(listActiveReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("分页查询关联码包列表")
    @GetMapping("listActivePackage")
    public BaseResponse<ListActivePackage> listActivePackage(@Validated GetActiveReq getActiveReq) throws Exception{
        //分页查询
        ListActivePackage result = activeService.listActivePackage(getActiveReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询活动明细")
    @GetMapping("getActive")
    public BaseResponse<GetActiveRes> getActive(@Validated GetActiveReq getActiveReq) throws Exception{
        //查询活动明细
        GetActiveRes  result = activeService.getActive(getActiveReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("活动启用")
    @PostMapping("enableActive")
    public BaseResponse enableActive(@Validated @RequestBody EnableActiveReq enableActiveReq) throws Exception{
        //活动启用
        activeService.enableActive(enableActiveReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("活动禁用")
    @PostMapping("disableActive")
    public BaseResponse disableActive(@Validated @RequestBody DisableActiveReq disableActiveReq) throws Exception{
        //活动禁用
        activeService.disableActive(disableActiveReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("审核通过")
    @PostMapping("/passActive")
    public BaseResponse passActive(@Validated @RequestBody PassActiveReq passActiveReq) {
        activeService.passActive(passActiveReq);
        return BaseResponse.create();
    }

    @ApiOperation("审核驳回")
    @PostMapping("/refuseActive")
    public BaseResponse refuseActive(@Validated @RequestBody RefuseActiveReq refuseActiveReq) {
        activeService.refuseActive(refuseActiveReq);
        return BaseResponse.create();
    }

}








