package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.service.MaterialService;
import com.abi.task.common.api.base.BaseResponse;
import com.abi.task.common.excel.ExcelInputFactory;
import com.abi.task.common.excel.preperties.DepartmentProperty;
import com.abi.task.common.excel.preperties.MaterialProperty;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 物料管理
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@RequestMapping("material")
@Api(tags = "物料管理")
public class MaterialController {

    @Autowired
    private MaterialService materialService;


    @ApiOperation("新增物料")
        @PostMapping("addMaterial")
    public BaseResponse addMaterial(@Validated @RequestBody AddMaterialReq addMaterialReq) throws Exception{
        //新增物料
        materialService.addMaterial(addMaterialReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("编辑物料")
    @PostMapping("updateMaterial")
    public BaseResponse updateMaterial(@Validated @RequestBody UpdateMaterialReq updateMaterialReq) throws Exception{
        //编辑物料
        materialService.updateMaterial(updateMaterialReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("分页查询物料")
    @GetMapping("listMaterial")
    public BaseResponse<ListMaterialRes> listMaterial(@Validated ListMaterialReq listMaterialReq) throws Exception{
        //分页查询物料
        ListMaterialRes result = materialService.listMaterial(listMaterialReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询物料明细")
    @GetMapping("getMaterial")
    public BaseResponse<GetMaterialRes> getMaterial(@Validated GetMaterialReq getMaterialReq) throws Exception{
        //查询物料明细
        GetMaterialRes  result = materialService.getMaterial(getMaterialReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("物料启用")
    @PostMapping("enableMaterial")
    public BaseResponse enableMaterial(@Validated @RequestBody EnableMaterialReq enableMaterialReq) throws Exception{
        //物料启用
        materialService.enableMaterial(enableMaterialReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("物料禁用")
    @PostMapping("disableMaterial")
    public BaseResponse disableMaterial(@Validated @RequestBody DisableMaterialReq disableMaterialReq) throws Exception{
        //物料禁用
        materialService.disableMaterial(disableMaterialReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("下拉选择物料列表")
    @GetMapping("listMaterialForSelect")
    public BaseResponse<ListMaterialForSelectRes> listMaterialForSelect(@Validated ListMaterialForSelectReq listMaterialForSelectReq) throws Exception{
        //下拉选择物料列表
        ListMaterialForSelectRes  result = materialService.listMaterialForSelect(listMaterialForSelectReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ResponseBody
    @ApiOperation(value = "导出模板")
    @GetMapping("downloadExcelModel")
    public void downloadExcelModel(HttpServletResponse response) throws Exception {
        ExcelInputFactory.downloadExcelModel("物料模板", response, MaterialProperty.class);
    }

    @ApiOperation("excel批量导入")
    @PostMapping("materialExcelImport")
    public BaseResponse<ExcelImportRes> materialExcelImport(@RequestParam("file") MultipartFile file) throws Exception {
        List<MaterialProperty> materialPropertyList = ExcelInputFactory.getExcelPojoList(file, MaterialProperty.class);
        //excel批量导入
        ExcelImportRes result = materialService.materialExcelImport(materialPropertyList);
        //包装出参
        return BaseResponse.create(result);
    }








}








