package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.CheckQrPackageRepetitiveRes;
import com.abi.qms.platform.dto.res.GetQrPackageDetailRes;
import com.abi.qms.platform.dto.res.ListQrPackageRes;
import com.abi.qms.platform.service.QrPackageService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;

/**
 * 码包 Controller
 *
 * @author WeiganCai
 * @date 2021-04-28
 */
@Slf4j
@RestController
@Api(tags = "码包管理")
@RequestMapping("/qrPackage")
public class QrPackageController {
    @Autowired
    private QrPackageService qrPackageService;

    @ApiOperation("码包新增修改")
    @PostMapping("/saveQrPackage")
    public BaseResponse saveQrPackage(@Validated @RequestBody SaveQrPackageReq saveQrPackageReq) {
        qrPackageService.saveQrPackage(saveQrPackageReq);
        return BaseResponse.create();
    }

    @ApiOperation("码包查询分页")
    @GetMapping("/listQrPackage")
    public BaseResponse<ListQrPackageRes> listQrPackage(@Validated ListQrPackageReq listQrPackageReq) {
        ListQrPackageRes result = qrPackageService.listQrPackage(listQrPackageReq);
        return BaseResponse.create(result);
    }

    @ApiOperation("码包查询详情")
    @GetMapping("/getQrPackageDetail")
    public BaseResponse<GetQrPackageDetailRes> getQrPackageDetail(@Validated GetQrPackageDetailReq getQrPackageDetailReq) {
        GetQrPackageDetailRes result = qrPackageService.getQrPackageDetail(getQrPackageDetailReq);
        return BaseResponse.create(result);
    }

    @ApiOperation("审核通过码包")
    @PostMapping("/passQrPackage")
    public BaseResponse passQrPackage(@Validated @RequestBody PassQrPackageReq passQrPackageReq) {
        qrPackageService.passQrPackage(passQrPackageReq);
        return BaseResponse.create();
    }

    @ApiOperation("审核拒绝码包")
    @PostMapping("/refuseQrPackage")
    public BaseResponse refuseQrPackage(@Validated @RequestBody RefuseQrPackageReq refuseQrPackageReq) {
        qrPackageService.refuseQrPackage(refuseQrPackageReq);
        return BaseResponse.create();
    }

    @ApiOperation("删除码包")
    @PostMapping("/deleteQrPackage")
    public BaseResponse deleteQrPackage(@Validated @RequestBody DeleteQrPackageReq deleteQrPackageReq) {
        qrPackageService.deleteQrPackage(deleteQrPackageReq);
        return BaseResponse.create();
    }

    @GetMapping("/sendMsg")
    @ApiOperation("发送短信")
    public BaseResponse sendMobileMsg(@Validated SendMobileMsgReq req) {
        //发送短信
        qrPackageService.sendMobileMsg(req);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("/下载码包")
    @GetMapping("/downloadQrPackage")
    public void downloadQrPackage(@Validated DownloadQrPackageReq downloadQrPackageReq, HttpServletResponse response) {
        qrPackageService.downloadQrPackage(downloadQrPackageReq, response);
    }

    @ApiOperation("校验是否存在SAP订单号和物料重复数据")
    @PostMapping("/validSAPOrderNoAndMaterial")
    public BaseResponse validSAPOrderNoAndMaterial(@Validated @RequestBody ValidSAPOrderNoAndMaterialReq req) {
        return BaseResponse.create(qrPackageService.validSAPOrderNoAndMaterial(req));
    }

}
