package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.ExcelImportRes;
import com.abi.qms.platform.dto.res.GetSkuRes;
import com.abi.qms.platform.dto.res.ListSkuForSelectRes;
import com.abi.qms.platform.dto.res.ListSkuRes;
import com.abi.qms.platform.service.SkuService;
import com.abi.task.common.api.base.BaseResponse;
import com.abi.task.common.excel.ExcelInputFactory;
import com.abi.task.common.excel.preperties.SkuProperty;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 商品管理
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@RequestMapping("sku")
@Api(tags = "商品管理")
public class SkuController {

    @Autowired
    private SkuService skuService;


    @ApiOperation("新增商品")
    @PostMapping("addSku")
    public BaseResponse addSku(@Validated @RequestBody AddSkuReq addSkuReq) throws Exception{
        //新增商品
        skuService.addSku(addSkuReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("编辑商品")
    @PostMapping("updateSku")
    public BaseResponse updateSku(@Validated @RequestBody UpdateSkuReq updateSkuReq) throws Exception{
        //编辑商品
        skuService.updateSku(updateSkuReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("分页查询商品")
    @GetMapping("listSku")
    public BaseResponse<ListSkuRes> listSku(@Validated ListSkuReq listSkuReq) throws Exception{
        //分页查询商品
        ListSkuRes result = skuService.listSku(listSkuReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询商品明细")
    @GetMapping("getSku")
    public BaseResponse<GetSkuRes> getSku(@Validated GetSkuReq getSkuReq) throws Exception{
        //查询商品明细
        GetSkuRes  result = skuService.getSku(getSkuReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("商品启用")
    @PostMapping("enableSku")
    public BaseResponse enableSku(@Validated @RequestBody EnableSkuReq enableSkuReq) throws Exception{
        //商品启用
        skuService.enableSku(enableSkuReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("商品禁用")
    @PostMapping("disableSku")
    public BaseResponse disableSku(@Validated @RequestBody DisableSkuReq disableSkuReq) throws Exception{
        //商品禁用
        skuService.disableSku(disableSkuReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("下拉选SKU列表")
    @GetMapping("listSkuForSelect")
    public BaseResponse<ListSkuForSelectRes> listSkuForSelect(@Validated ListSkuForSelectReq listSkuForSelectReq) throws Exception{
        //下拉选择物料列表
        ListSkuForSelectRes  result = skuService.listSkuForSelect(listSkuForSelectReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ResponseBody
    @ApiOperation(value = "导出模板")
    @GetMapping("downloadExcelModel")
    public void downloadExcelModel(HttpServletResponse response) throws Exception {
        ExcelInputFactory.downloadExcelModel("SKU模板", response, SkuProperty.class);
    }

    @ApiOperation("excel批量导入")
    @PostMapping("skuExcelImport")
    public BaseResponse<ExcelImportRes> skuExcelImport(@RequestParam("file") MultipartFile file) throws Exception {
        List<SkuProperty> skuPropertyList = ExcelInputFactory.getExcelPojoList(file, SkuProperty.class);
        //excel批量导入
        ExcelImportRes result = skuService.skulExcelImport(skuPropertyList);
        //包装出参
        return BaseResponse.create(result);
    }

}








