package com.abi.qms.platform.infrastructure.mq;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.file.FileWriter;
import com.abi.base.foundation.util.JsonUtil;
import com.abi.base.foundation.util.RedisClient;
import com.abi.qms.platform.dao.entity.*;
import com.abi.qms.platform.dao.enums.QrFormatUrlEnum;
import com.abi.qms.platform.dao.enums.QrPackageGenerateStatusEnum;
import com.abi.qms.platform.dao.enums.QrRepertoryTypeEnum;
import com.abi.qms.platform.dao.mapper.*;
import com.abi.qms.platform.dao.tablestore.entity.QrCode;
import com.abi.qms.platform.dao.vo.result.QrPackageVO;
import com.abi.qms.platform.dao.vo.result.QrRepertoryColumnVO;
import com.abi.qms.platform.dao.vo.result.QrRepertoryVO;
import com.abi.qms.platform.feign.BasicServiceClient;
import com.abi.qms.platform.infrastructure.qr.build.parent.SerialBuildCode;
import com.abi.qms.platform.infrastructure.util.BuildCodeUtil;
import com.abi.qms.platform.infrastructure.util.RandomCodeUtils;
import com.abi.qms.platform.service.QrPackageService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.tablestore.TableStorePlusUtils;
import com.abi.task.common.tablestore.common.TableStoreEntity;
import com.abi.task.common.tablestore.entity.DemoEntity;
import com.abi.task.common.utils.TableStoreUtils;
import com.alicloud.openservices.tablestore.model.search.query.BoolQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.rabbitmq.client.Channel;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;
import net.lingala.zip4j.core.ZipFile;
import net.lingala.zip4j.exception.ZipException;
import net.lingala.zip4j.model.ZipParameters;
import net.lingala.zip4j.util.Zip4jConstants;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 生成码-消费者
 *
 * @author WeiganCai
 * @date: 2021-05-06
 */
@Slf4j
@Component
public class GenerateCodeConsumer {

	public static final String GENERATE_CODE_BY_REPERTORY_QUEUE = "generate_code_queue";

	@Autowired
	private QrPackageMapper qrPackageMapper;

	@Autowired
	private QrRepertoryMapper qrRepertoryMapper;

	@Autowired
	private QrRepertorySerialNumberMapper qrRepertorySerialNumberMapper;

	@Autowired
	private QrPackageService qrPackageService;

	@Autowired
	private RedissonClient redissonClient;

	@Autowired
	private TableStorePlusUtils tableStorePlusUtils;

	@Autowired
    private QrPackageBatchMapper qrPackageBatchMapper;

	@Autowired
	private BasicServiceClient basicServiceClient;

	@Autowired
	private BaseFactoryMapper baseFactoryMapper;

	@Autowired
	private QrBoxCodeFormatMapper boxCodeFormatMapper;

	@Autowired
	private QrBoxCodeFormatSplitMapper qrBoxCodeFormatSplitMapper;

	@Autowired
	private QrIndexMapper qrIndexMapper;

	@Autowired
	private RedisClient redisClient;

	//生成码数据
	final List<QrData> qrDataList = new LinkedList<>();

	//生成码包路径
	private File file = new File(this.getClass().getResource("/").getPath()+"/code");

	//批量添加文件
	private ArrayList<File> files = new ArrayList<>();

	/** 允许单个码生成重复的次数 */
	private static final int REPEAT_BUILD_CODE_TIMES = 10;

	/** 一次生成数量 */
	public static final int ONCE_GENERATE_NUMBER = 5000;

	/** 声明码对应的值 */
	public Long codeIndex = 0L;

	/** 系统url */
	@Value("${system.url}")
	private String systemUrl;

	@SneakyThrows
	@RabbitHandler
	@RabbitListener(queues = GenerateCodeConsumer.GENERATE_CODE_BY_REPERTORY_QUEUE)
	public void onMessage(String id, Message message, Channel channel) {
        String str = JsonUtil.toBean(id, String.class);
		QrPackageVO qrPackageVO = qrPackageMapper.selectQrPackageDetailById(Long.parseLong(str));
		try {
			Object code_qr_index = redisClient.get("code_qr_index");
			if(ObjectUtils.isEmpty(code_qr_index)){
				QueryWrapper<QrIndex>  indexQuery= new QueryWrapper<>();
				indexQuery.orderByDesc("index_current").last("limit 1");
				QrIndex qrIndex = qrIndexMapper.selectOne(indexQuery);
				if(!ObjectUtils.isEmpty(qrIndex)){
					codeIndex = qrIndex.getIndexCurrent();
				}
				redisClient.set("code_qr_index",codeIndex);
			}
			// 生成码
			loopGenerateCode(qrPackageVO);
		} catch (Exception e) {
			log.error("生成码失败", e);
			// 回滚
			QrPackage qrPackage = new QrPackage();
			qrPackage.setId(qrPackageVO.getId());
			qrPackage.setGenerateStatus(qrPackageVO.getGenerateStatus());
			qrPackageService.failedRollback(qrPackage);
		} finally {
			// 手工ack
			Long deliveryTag = (Long)message.getHeaders().get(AmqpHeaders.DELIVERY_TAG);
			channel.basicAck(deliveryTag, false);
		}
	}

	/**
	 * 生成码
	 */
	private void loopGenerateCode(QrPackageVO qrPackageVO) {
		//刷新码包信息
		QrPackage qrPackage = qrPackageMapper.selectById(qrPackageVO.getId());

		// 查询码包关联的幅面关联的码库的信息
		QrPackageVO qrPackageVo = qrPackageMapper.selectQrPackageDetailById(qrPackage.getId());

		// 码数据已生成，return
		if (QrPackageGenerateStatusEnum.GENERATE_SUCCESS.is(qrPackageVo.getGenerateStatus())) {
			return;
		}

		// 更改生成状态为生成中
		qrPackage.setGenerateStatus(QrPackageGenerateStatusEnum.GENERATING.getCode());
		qrPackageMapper.updateById(qrPackage);

		// 码库 & 码库的列
		QrRepertoryVO qrRepertory = qrRepertoryMapper.selectQrRepertoryDetailById(qrPackageVo.getQrRepertoryId());
		List<QrRepertoryColumnVO> qrRepertoryColumnList = qrRepertory.getQrRepertoryColumnList();
		Long qrRepertoryId = qrRepertory.getId();

		// 存放每列的url
		Map<Long, String> urlMap = new LinkedHashMap<>();

		// key:qr_repertory_column_id + child_index value:QrRepertorySerialNumber，后续会更新流水号
		// child_index用于标记同一列的第几个码格式变量
		Map<String, QrRepertorySerialNumber> qrRepertorySerialNumberMap = new LinkedHashMap<>();

		// 处理url和流水号--后续生成码要用到
		getUrlAndSerialNumber(qrRepertoryColumnList, qrRepertoryId, urlMap, qrRepertorySerialNumberMap);

		//码包id
        //根据码包id查询,获取批次号
		QueryWrapper<QrPackageBatch> param = new QueryWrapper<>();
		param.eq("package_id", qrPackageVo.getId());
		List<QrPackageBatch> qrPackageBatches = qrPackageBatchMapper.selectList(param);

		//1-生成二维码
		for (QrPackageBatch qrPackageBatch:qrPackageBatches) {
			// 这批下的申请数量
			Long qrNumber = qrPackageBatch.getBatchQrNumber();
			//批次号
			String batchNumber = qrPackageBatch.getBatchNumber();
			//重复次数
			int repeatTimes = 0;
			//获取箱码幅面
			QrBoxCodeFormat boxCodeFormat = boxCodeFormatMapper.selectById(qrPackageVo.getBoxCodeFormatId());

			/**
			 * qrRepertoryColumnList  码库的列
			 * qrRepertoryId 码库id
			 * urlMap 存放每列的url
			 * qrRepertorySerialNumberMap 用于标记同一列的第几个码格式变量
			 * qrNumber 码数量
			 * batchNumber 批次号
			 * localDateTime 创建时间
			 * boxCodeFormat 箱码幅面
			 */
			// 循环生成码
			List<QrData> codeList = loopGenerateCode(qrRepertoryColumnList, qrRepertoryId, urlMap, qrRepertorySerialNumberMap, qrNumber, batchNumber, repeatTimes,boxCodeFormat);

            // 更新码库流水号
            if (!qrRepertorySerialNumberMap.isEmpty()) {
                for (Map.Entry<String, QrRepertorySerialNumber> entry : qrRepertorySerialNumberMap.entrySet()) {
                    QrRepertorySerialNumber qrRepertorySerialNumber = entry.getValue();
                    QrRepertorySerialNumber updateSerialNumber = new QrRepertorySerialNumber()
                            .setId(qrRepertorySerialNumber.getId())
                            .setSerialNumber(qrRepertorySerialNumber.getSerialNumber());
                    qrRepertorySerialNumberMapper.updateById(updateSerialNumber);
                }
            }

			// table数据库  批量插入
			saveTableStore(qrPackageBatch, codeList);

			// 创建码文件
			createCodeFile(qrRepertory,batchNumber,boxCodeFormat);

		}

		//2-保存压缩文件并上传oss
		//压缩后得名字
		String zipName = DateTimeFormatter.ofPattern("yyyyMMddHHmmss").format(LocalDateTime.now())+".zip";
		//压缩密码
		String pas = RandomCodeUtils.getStr(6);
		log.info("生成密码："+pas);
		//压缩加密文件
		zipEncryption(file,files,pas,zipName);
		//上传阿里云，并更新码包的下载路径 保存zip密码
		try{
			MultipartFile multipartFile = toMultipartFile(file.getPath()+"\\"+zipName, zipName);
			Object returDat = basicServiceClient.uploadFileOss(multipartFile);
			if(null != returDat){
				Map<String,Object> result =(Map<String, Object>)returDat;
				//放入oss地址
				qrPackage.setDownloadPath(null ==  result.get("data") ? "" :(String) result.get("data"));
			}
			//放入oss地址和密码
			qrPackage.setZipPassword(pas);
			//删除生成文件
			FileUtil.del(file);
		}catch (Exception e){
			throw new BusinessException("上传OSS失败");
		}

		//3-更改生成状态为已生成
		qrPackage.setGenerateStatus(QrPackageGenerateStatusEnum.GENERATE_SUCCESS.getCode());
		qrPackage.setGenerateTime(LocalDateTime.now());
		//根据包材厂获取包材厂负责人邮箱
		QueryWrapper<BaseFactory> baseQuery = new QueryWrapper<>();
		baseQuery.eq("id", qrPackage.getFactoryCoverId());
		BaseFactory baseFactory = baseFactoryMapper.selectOne(baseQuery);
		//发送邮件
		String aliPath = qrPackage.getDownloadPath().substring(qrPackage.getDownloadPath().lastIndexOf("/") + 1);
		qrPackageService.sendEmailForQrPackageCreated(baseFactory.getEmail(),qrPackage.getSapOrderNo(),aliPath,qrPackage.getZipPassword());
		//邮件发送后修改状态
		qrPackageMapper.updateById(qrPackage);
		QrIndex index = new QrIndex();
		index.setIndexCurrent(codeIndex);
		qrIndexMapper.insert(index);
		redisClient.set("code_qr_index",codeIndex);
		codeIndex = 0L;
		qrDataList.clear();
	}

	/**
	 * 转换成 MultipartFile
	 * @param filePath
	 * @param fileName
	 * @return
	 */
	private MultipartFile toMultipartFile(String filePath, String fileName){

		File file = new File(filePath);

		FileItemFactory factory = new DiskFileItemFactory(16, null);
		FileItem fileItem = factory.createItem(fileName, "text/plain", true, file.getName());
		int bytesRead = 0;
		byte[] buffer = new byte[8192];
		try {
			FileInputStream fis = new FileInputStream(file);
			OutputStream os = fileItem.getOutputStream();
			while ((bytesRead = fis.read(buffer, 0, 8192)) != -1) {
				os.write(buffer, 0, bytesRead);
			}
			os.close();
			fis.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		MultipartFile multipartFile = new CommonsMultipartFile(fileItem);
		return multipartFile;
	}

	/**
	 * 批量新增tableStore,码表
	 */
	private void saveTableStore(QrPackageBatch qrPackageBatch, List<QrData> codeList) {
		List<TableStoreEntity> codeSave = new ArrayList<>();
		for (QrData qrData:codeList) {
			List<QrInnerData> innerDataList = qrData.getInnerDataList();
			for (QrInnerData qrInnerData:innerDataList) {
				QrCode qrCode = new QrCode();
				qrCode.setId(String.valueOf(IdWorker.getId()));
				//码
				qrCode.setCode(qrInnerData.getCode());
				//批次id
				qrCode.setBatchNumberId(qrPackageBatch.getId());
				//码包id
				qrCode.setPackageId(qrPackageBatch.getPackageId());
				//要拼接的url
				qrCode.setUrl(qrInnerData.getUrl());
				//创建时间
				qrCode.setCreateTime(LocalDateTime.now());
				//已生成码包位置
				qrCode.setCodeIndex(qrInnerData.getCodeIndex());
				codeSave.add(qrCode);
			}
		}
		tableStorePlusUtils.putRow(codeSave);
	}

	/**
     * qrRepertoryColumnList  码库的列
     * qrRepertoryId 码库id
     * urlMap 存放每列的url
     * qrRepertorySerialNumberMap 用于标记同一列的第几个码格式变量
     * qrNumber 码数量
     * batchNumber 批次号
     * localDateTime 创建时间
     * repeatTimes 失败次数
	 * formatNumber 幅面数量
     */
	private List<QrData> loopGenerateCode(List<QrRepertoryColumnVO> qrRepertoryColumnList, Long qrRepertoryId, Map<Long, String> urlMap, Map<String, QrRepertorySerialNumber> qrRepertorySerialNumberMap, Long qrNumber, String batchNumber, int repeatTimes,QrBoxCodeFormat boxCodeFormat ) {
        LocalDateTime localDateTime = LocalDateTime.now();
		while (true) {
			// 计算还需生成的数量
			long needGenerateNumber = qrNumber --;

			if (needGenerateNumber <= 0) {
				log.info("批次号：{}，码生成完毕", batchNumber);
				break;
			}

			// 计算本次最多生成多少码
			long currentMaxGenerateNumber = needGenerateNumber <= ONCE_GENERATE_NUMBER ? needGenerateNumber : ONCE_GENERATE_NUMBER;

			// 用Set存放不重复的码
			HashSet<String> uniqueSet = new HashSet<>();
			// 生成码要用的参数
			Map<String, Object> paramMap = new HashMap<>();
			// 码
			StringBuilder codeSb = new StringBuilder();

			for (int i = 0; i < currentMaxGenerateNumber; i++) {
				QrData qrData = new QrData()
						.setQrRepertoryId(qrRepertoryId)
						.setBatchNumber(batchNumber)
						.setQrRepertoryType(QrRepertoryTypeEnum.SYSTEM_GENERATE.getCode())
						.setCreateTime(localDateTime);

				List<QrInnerData> qrInnerDataList = new LinkedList<>();
				//箱子码幅面
				for (int a = 0; a < boxCodeFormat.getFormatNumber(); a++) {
					for (int j = 0; j < qrRepertoryColumnList.size(); j++) {
						QrRepertoryColumnVO qrRepertoryColumn = qrRepertoryColumnList.get(j);
						Long qrRepertoryColumnId = qrRepertoryColumn.getId();
						//判断是否有拆分数量  0 或者空不拆分 ，根据箱码幅面id，码库id，码库列id 查询拆分数量
						QueryWrapper<QrBoxCodeFormatSplit> boxCodeFormatSplit = new QueryWrapper<>();
						boxCodeFormatSplit.eq("box_code_format_id",boxCodeFormat.getId() );
						boxCodeFormatSplit.eq("qr_repertory_id", boxCodeFormat.getQrRepertoryId());
						boxCodeFormatSplit.eq("qr_repertory_column_id",qrRepertoryColumnId );
						QrBoxCodeFormatSplit qrBoxCodeFormatSplits = qrBoxCodeFormatSplitMapper.selectOne(boxCodeFormatSplit);
						//声明拆分数量
						int splitNum = 0;
						//判断是否为空
						if(ObjectUtils.isNotEmpty(qrBoxCodeFormatSplits)){
							//查分数量
							splitNum = qrBoxCodeFormatSplits.getSplitNum();
						}

						// 非clone列，设置code和url
						String code = buildUniqueCode(uniqueSet, qrRepertoryColumn, codeSb, paramMap, qrRepertorySerialNumberMap, 0);
						String url = urlMap.get(qrRepertoryColumnId);
						if (code == null) {
							throw new RuntimeException("生成码失败，重复生成码次数超过" + REPEAT_BUILD_CODE_TIMES + "次");
						}
						//判断拆分数量是否为空
						if( splitNum >0){
							List<char[]> codeStr = a(code, splitNum);
							for (char[] chars:codeStr) {
								QrInnerData qrInnerData = new QrInnerData();
								qrInnerData.setCode(String.valueOf(chars)).setUrl(url);
								qrInnerData.setQrRepertoryColumnId(qrRepertoryColumnId).setSortNumber(qrRepertoryColumn.getSortNumber());
								qrInnerData.setCodeIndex(codeIndex+=1);
								qrInnerDataList.add(qrInnerData);
							}
						} else {
							QrInnerData qrInnerData = new QrInnerData();
							qrInnerData.setCode(code).setUrl(url);
							qrInnerData.setQrRepertoryColumnId(qrRepertoryColumnId).setSortNumber(qrRepertoryColumn.getSortNumber());
							qrInnerData.setCodeIndex(codeIndex+=1);
							qrInnerDataList.add(qrInnerData);
						}
					}
				}
				qrData.setInnerDataList(qrInnerDataList);
				qrDataList.add(qrData);

			}

			try {
				return qrDataList;
			} catch (Exception e) {
				log.error("封装码失败", e);
				if (++repeatTimes >= REPEAT_BUILD_CODE_TIMES) {
					throw new RuntimeException("生成码失败，重复生成码次数超过" + REPEAT_BUILD_CODE_TIMES + "次");
				}
			}
		}
		return null;
	}

	/**
	 * 处理url和流水号
	 */
	private void getUrlAndSerialNumber(List<QrRepertoryColumnVO> qrRepertoryColumnList, Long qrRepertoryId, Map<Long, String> urlMap, Map<String, QrRepertorySerialNumber> qrRepertorySerialNumberMap) {
		for (int i = 0; i < qrRepertoryColumnList.size(); i++) {
			QrRepertoryColumnVO qrRepertoryColumn = qrRepertoryColumnList.get(i);
			Long qrRepertoryColumnId = qrRepertoryColumn.getId();

			// 每列的url格式
			Integer urlType = qrRepertoryColumn.getUrlType();
			String url = "";
			if (QrFormatUrlEnum.SYSTEM_URL.is(urlType)) {
				url = systemUrl;
			}
			// 如果url没有以/结尾，则加上/
			if (StringUtils.isNotBlank(url) && !url.endsWith("/")) {
				url += "/";
			}
			urlMap.put(qrRepertoryColumnId, url);

			// 使用到流水号参数的列
			String[] buildClassArr = qrRepertoryColumn.getQrFormatVO().getCodeVariableBuildClass().split(",");
			for (int j = 0; j < buildClassArr.length; j++) {
				String buildClass = buildClassArr[j];
				if (buildClass.startsWith(SerialBuildCode.SERIAL_START_WITH)) {
					// 查询码库流水号，若存在，则流水号从记录值开始；若不存在，从0开始
					QueryWrapper<QrRepertorySerialNumber> qrRepertorySerialNumberQw = new QueryWrapper<>();
					qrRepertorySerialNumberQw.eq("qr_repertory_column_id", qrRepertoryColumnId);
					qrRepertorySerialNumberQw.eq("child_index", j);

					// 加锁查询插入流水号
					RLock lock = redissonClient.getLock("serial_number_lock_" + qrRepertoryColumnId + "_" + j);
					QrRepertorySerialNumber qrRepertorySerialNumber = null;
					try {
						lock.lock();
						qrRepertorySerialNumber = qrRepertorySerialNumberMapper.selectOne(qrRepertorySerialNumberQw);
						if (qrRepertorySerialNumber == null) {
							qrRepertorySerialNumber = new QrRepertorySerialNumber()
									.setQrRepertoryId(qrRepertoryId)
									.setQrRepertoryColumnId(qrRepertoryColumnId)
									.setChildIndex(j)
									.setSerialBuildClass(buildClass)
									.setSerialNumber(0L);
							qrRepertorySerialNumberMapper.insert(qrRepertorySerialNumber);
						}
					} finally {
						lock.unlock();
					}

					// key:qr_repertory_column_id + child_index
					qrRepertorySerialNumberMap.put(qrRepertoryColumnId + "_" + j, qrRepertorySerialNumber);
				}
			}

		}
	}

	/**
	 * 创建码文件
	 */
	private void createCodeFile( QrRepertoryVO qrRepertory,String batchNumber,QrBoxCodeFormat boxCodeFormat) {
		List<QrRepertoryColumnVO> qrRepertoryColumnList = qrRepertory.getQrRepertoryColumnList();
		StringBuilder content = new StringBuilder();
		// 第一行为每列的名称，用“,”分隔
		String dataComposition = qrRepertoryColumnList.stream().map(col -> col.getAlias()).collect(Collectors.joining(","));
        content.append(dataComposition);
        //暂时根据项目幅面分，没有根据拆分数量分
		if(boxCodeFormat.getFormatNumber() > 1){
			for (int i = 1; i < boxCodeFormat.getFormatNumber(); i++) {
				content.append(",").append(dataComposition);
			}
		}
		content.append("\r\n");
		// 拼接码
		List<QrInnerData> innerDataList;
		String innerData;
		for (QrData qrData : qrDataList) {
			innerDataList = qrData.getInnerDataList();
			innerData = innerDataList.stream().map(qrInnerData -> qrInnerData.getUrl() + qrInnerData.getCode()).collect(Collectors.joining(","));
			content.append(innerData).append("\r\n");
		}
		//如果文件不存在，创建一个文件
		try{
			if (!file.exists()) {
				file.mkdir();
			}
		}catch (Exception e){
			log.error("创建文件异常",e);
		}

		// 生成txt
		FileWriter fileWriter = new FileWriter(file.getPath()+"/"+ batchNumber + ".txt");
		fileWriter.write(content.toString());
		files.add(new File(file.getPath()+"/"+ batchNumber + ".txt"));
	}

	/**
	 * 生成不重复的码
	 */
	private String buildUniqueCode( HashSet<String> uniqueSet, QrRepertoryColumnVO qrRepertoryColumn, StringBuilder codeSb, Map<String, Object> paramMap, Map<String, QrRepertorySerialNumber> qrRepertorySerialNumberMap, int buildTimes) {
		// 生成码
		buildCodeByClass(qrRepertoryColumn, codeSb, paramMap, qrRepertorySerialNumberMap);
		String code = codeSb.toString();

		// add成功，说明码没有重复
		if (uniqueSet.add(code)) {
			return code;
		}

		// 单个码重复次数大于上限
		if (buildTimes >= REPEAT_BUILD_CODE_TIMES) {
			return null;
		}

		// 码重复，递归执行下一次
		return buildUniqueCode(uniqueSet, qrRepertoryColumn, codeSb, paramMap, qrRepertorySerialNumberMap, ++buildTimes);
	}

	/**
	 * 清空codeSb，调用buildSingleCode方法生成码
	 */
	private void buildCodeByClass(QrRepertoryColumnVO qrRepertoryColumn, StringBuilder codeSb, Map<String, Object> paramMap, Map<String, QrRepertorySerialNumber> qrRepertorySerialNumberMap) {
		// 使用一个对象，删除内容，不每次都新建一个对象
		codeSb.delete(0, codeSb.length());

		String[] buildClassArr = qrRepertoryColumn.getQrFormatVO().getCodeVariableBuildClass().split(",");

		for (int j = 0; j < buildClassArr.length; j++) {
			String buildClass = buildClassArr[j];
			String code;
			if (buildClass.startsWith(SerialBuildCode.SERIAL_START_WITH)) {
				QrRepertorySerialNumber qrRepertorySerialNumber = qrRepertorySerialNumberMap.get(qrRepertoryColumn.getId() + "_" + j);
				// 查询当前流水号的值，放入map中
				paramMap.put(buildClass, qrRepertorySerialNumber.getSerialNumber());
				code = BuildCodeUtil.buildSingleCode(buildClass, paramMap);
				// 重新赋值
				qrRepertorySerialNumber.setSerialNumber(Long.parseLong(code));
			} else {
				code = BuildCodeUtil.buildSingleCode(buildClass, paramMap);
			}
			codeSb.append(code);
		}
	}

	/**
	 * zip加密
	 * @param file
	 * @param files txt文件
	 * @param pas 密码
	 * @param zipName 压缩后的名字
	 */
	private void zipEncryption(File file  ,ArrayList<File> files,String pas,String zipName){
		try {
			//创建压缩文件
			ZipFile zipFile = new ZipFile(file.getPath()+"/"+zipName);
			//设置压缩文件参数
			ZipParameters parameters = new ZipParameters();
			//设置压缩方法
			parameters.setCompressionMethod(Zip4jConstants.COMP_DEFLATE);

			//设置压缩级别
			//DEFLATE_LEVEL_FASTEST     - Lowest compression level but higher speed of compression
			//DEFLATE_LEVEL_FAST        - Low compression level but higher speed of compression
			//DEFLATE_LEVEL_NORMAL  - Optimal balance between compression level/speed
			//DEFLATE_LEVEL_MAXIMUM     - High compression level with a compromise of speed
			//DEFLATE_LEVEL_ULTRA       - Highest compression level but low speed
			parameters.setCompressionLevel(Zip4jConstants.DEFLATE_LEVEL_NORMAL);

			//设置压缩文件加密
			parameters.setEncryptFiles(true);

			//设置加密方法
			parameters.setEncryptionMethod(Zip4jConstants.ENC_METHOD_AES);

			//设置aes加密强度
			parameters.setAesKeyStrength(Zip4jConstants.AES_STRENGTH_256);

			//设置密码
			parameters.setPassword(pas);

			//添加文件到压缩文件
			zipFile.addFiles(files, parameters);
		} catch (ZipException e) {
			e.printStackTrace();
		}
	}


	public static List<char[]> a(String str, int groupSum){
		List<char[]> result = new ArrayList<>();

		if (null == str){
			return null;
		}

		if (groupSum < 0 || str.length() < groupSum){
			result.add(str.toCharArray());
		}else {

			int group = str.length() / groupSum ;

			int count = 0;

			while (true){
				if (result.size() + 1 == groupSum){
					result.add(str.substring(count * group).toCharArray());
					break;
				}
				result.add(str.substring(count * group ,count * group + group ).toCharArray());
				count ++ ;
			}

		}

		return result;
	}
}
