package com.abi.qms.platform.service;


import com.abi.qms.platform.dao.entity.QrPackage;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.CheckQrPackageRepetitiveRes;
import com.abi.qms.platform.dto.res.GetQrPackageDetailRes;
import com.abi.qms.platform.dto.res.ListQrPackageRes;
import com.abi.qms.platform.dto.res.ValidSAPOrderNoAndMaterialRes;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;

/**
 * 码包 Service接口
 *
 * @author WeiganCai
 * @date 2021-04-28
 */
public interface QrPackageService {

	/**
	 * 保存码包
	 */
	void saveQrPackage(SaveQrPackageReq saveQrPackageReq);

	/**
	 * 码包分页查询
	 */
	ListQrPackageRes listQrPackage(ListQrPackageReq listQrPackageReq);

	/**
	 * 码包查询详情
	 */
	GetQrPackageDetailRes getQrPackageDetail(GetQrPackageDetailReq getQrPackageDetailReq);

	/**
	 * 删除码包
	 */
	void deleteQrPackage(DeleteQrPackageReq deleteQrPackageReq);

	/**
	 * 审核通过码包
	 */
	void passQrPackage(PassQrPackageReq passQrPackageReq);

	/**
	 * 审核拒绝码包
	 */
	void refuseQrPackage(RefuseQrPackageReq refuseQrPackageReq);

	/**
	 * 下载码包
	 */
	void downloadQrPackage(DownloadQrPackageReq downloadQrPackageReq, HttpServletResponse response);

	/**
	 * 码生成失败回滚
	 */
	void failedRollback(QrPackage qrPackage);

	/**
	 * 码包生成完成后发送邮件给码包厂负责人邮箱
	 * @param targetEmail
	 * @param sapOrderNo
	 * @param filename
	 * @param password
	 */
	void sendEmailForQrPackageCreated(String targetEmail,String sapOrderNo,String filename,String password);

	/**
	 * 校验是否存在SAP订单号和物料重复数据
	 * @param req
	 * @return
	 */
	ValidSAPOrderNoAndMaterialRes validSAPOrderNoAndMaterial(ValidSAPOrderNoAndMaterialReq req);

	/**
	 * 发送短信验证码
	 * @param req
	 */
	void sendMobileMsg(SendMobileMsgReq req);



}
