package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.qms.platform.dao.entity.BaseFactory;
import com.abi.qms.platform.dao.entity.QrBoxCodeFormat;
import com.abi.qms.platform.dao.entity.QrPackage;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.dao.enums.FactoryTypeEnum;
import com.abi.qms.platform.dao.enums.LoginTypeEnum;
import com.abi.qms.platform.dao.enums.ValidEnum;
import com.abi.qms.platform.dao.mapper.BaseFactoryMapper;
import com.abi.qms.platform.dao.mapper.QrBoxCodeFormatMapper;
import com.abi.qms.platform.dao.mapper.QrPackageMapper;
import com.abi.qms.platform.dao.mapper.UserInfoMapper;
import com.abi.qms.platform.dto.req.DisableFactoryReq;
import com.abi.qms.platform.dto.req.EnableFactoryReq;
import com.abi.qms.platform.dto.req.GetFactoryDetailReq;
import com.abi.qms.platform.dto.req.ListFactoryReq;
import com.abi.qms.platform.dto.req.SaveFactoryReq;
import com.abi.qms.platform.dto.res.ExcelImportRes;
import com.abi.qms.platform.dto.res.GetFactoryDetailRes;
import com.abi.qms.platform.dto.res.ListBeerFactorySelectRes;
import com.abi.qms.platform.dto.res.ListFactoryEnableRes;
import com.abi.qms.platform.dto.res.ListFactoryRes;
import com.abi.qms.platform.dto.res.ListPackageFactorySelectRes;
import com.abi.qms.platform.dto.res.ListPageMaterialsRes;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.FactoryService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.excel.preperties.FactoryProperty;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 供应商管理
 * </p>
 *
 * @author Andy.Tan
 */
@Service
public class FactoryServiceImpl implements FactoryService {

    @Autowired
    private BaseFactoryMapper baseFactoryMapper;

    @Autowired
    private UserInfoMapper userInfoMapper;

    @Autowired
    private QrBoxCodeFormatMapper qrBoxCodeFormatMapper;

    @Autowired
    private QrPackageMapper qrPackageMapper;

    @Autowired
    private UserUtil userUtil;

    /**
     * 供应商新增修改
     **/
    @Override
    public void saveFactory(SaveFactoryReq req) {

        //0-校验入参
        //code不能重复
        QueryWrapper<BaseFactory> factoryQw = new QueryWrapper<>();
        factoryQw.eq("factory_code", req.getFactoryCode());
        if (req.getId() != null) {
            factoryQw.ne("id", req.getId());
        }
        List<BaseFactory> checkFactoryList = baseFactoryMapper.selectList(factoryQw);
        if (CollectionUtil.isNotEmpty(checkFactoryList)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "供应商ID已存在");
        }

        //1-声明对象
        BaseFactory dept = null;
        if (req.getId() != null) {
            dept = baseFactoryMapper.selectById(req.getId());
            AssertUtil.isNull(dept, "供应商不存在");
        } else {
            dept = new BaseFactory();
        }

        //2-放入数据
        dept.setFactoryCode(req.getFactoryCode());
        dept.setFactoryName(req.getFactoryName());
        dept.setFactoryType(req.getFactoryType());
        dept.setDescription(req.getDescription());
        if (FactoryTypeEnum.COVER.is(req.getFactoryType())) {
            dept.setKeymanName(req.getKeymanName());
            dept.setKeymanMobile(req.getKeymanMobile());
            dept.setEmail(req.getEmail());
            dept.setMaxFileCount(req.getMaxFileCount());
            dept.setFactoryLevel(req.getFactoryLevel());
            dept.setProvince(req.getProvince());
            dept.setCity(req.getCity());
            dept.setDistrict(req.getDistrict());
            dept.setAddress(req.getAddress());
        }

        //3-新增or修改
        if (req.getId() != null) {
            baseFactoryMapper.updateById(dept);
        } else {
            baseFactoryMapper.insert(dept);
        }

    }

    /**
     * 供应商查询分页
     **/
    @Override
    public ListFactoryRes listFactory(ListFactoryReq req) {
        //1-拼接条件
        QueryWrapper<BaseFactory> factoryQw = new QueryWrapper<>();
        factoryQw.like(StringUtils.isNotBlank(req.getFactoryName()), "factory_name", "%" + req.getFactoryName() + "%");
        factoryQw.eq(req.getFactoryType() != null, "factory_type", req.getFactoryType());
        factoryQw.eq(req.getValid() != null, "valid", req.getValid());

        //2-分页查询
        IPage<BaseFactory> iPage = baseFactoryMapper.selectPage(PageUtil.createPage(req), factoryQw);
        List<BaseFactory> factoryList = iPage.getRecords();

        //封装出参
        ListFactoryRes res = new ListFactoryRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListFactoryRes.FactoryBean> factoryBeanList = PojoConverterUtils.copyList(factoryList, ListFactoryRes.FactoryBean.class);
        res.setFactoryBeanList(factoryBeanList);

        return res;
    }

    /**
     * 供应商查询详情
     **/
    @Override
    public GetFactoryDetailRes getFactoryDetail(GetFactoryDetailReq req) {
        //1-查询供应商对象
        BaseFactory factory = baseFactoryMapper.selectById(req.getId());
        AssertUtil.isNull(factory, "供应商不存在");

        //转化出参
        GetFactoryDetailRes res = PojoConverterUtils.copy(factory, GetFactoryDetailRes.class);

        return res;
    }

    /**
     * 供应商启用
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void enableFactory(EnableFactoryReq req) {
        List<Long> ids = req.getIds();

        //循环启用
        for (Long id : ids) {
            //1-查询活动对象
            BaseFactory factory = baseFactoryMapper.selectById(id);
            AssertUtil.isNull(factory, "供应商不存在");
            if (!ValidEnum.NOT_VALID.is(factory.getValid())) {
                continue;
            }

            //2-修改状态
            factory.setValid(ValidEnum.VALID.getCode());
            baseFactoryMapper.updateById(factory);
        }
    }

    /**
     * 供应商禁用
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void disableFactory(DisableFactoryReq req) {
        List<Long> ids = req.getIds();

        //循环禁用
        for (Long id : ids) {
            //1-查询活动对象
            BaseFactory factory = baseFactoryMapper.selectById(id);
            AssertUtil.isNull(factory, "供应商不存在");
            if (!ValidEnum.VALID.is(factory.getValid())) {
                return;
            }

            //只要和当前供应商绑定的（包含启用禁用状态），都不能禁用（码包，箱码幅面，用户)
            //用户
            QueryWrapper<UserInfo> userQuery = new QueryWrapper<>();
            userQuery.eq("factory_id", id);
            userQuery.eq("is_delete", 0);
            List<UserInfo> userInfos = userInfoMapper.selectList(userQuery);
            if (CollectionUtil.isNotEmpty(userInfos)) {
                throw new BusinessException("供应商" + factory.getFactoryName() + "已与用户" + userInfos.get(0).getUserName() + "绑定，请解绑后再停用。");
            }
            //箱码幅面
            QueryWrapper<QrBoxCodeFormat> boxCodeFormatQuery = new QueryWrapper<>();
            boxCodeFormatQuery.eq("factory_id", id);
            boxCodeFormatQuery.eq("is_delete", 0);
            List<QrBoxCodeFormat> qrBoxCodeFormats = qrBoxCodeFormatMapper.selectList(boxCodeFormatQuery);
            if (CollectionUtil.isNotEmpty(qrBoxCodeFormats)) {
                throw new BusinessException("供应商" + factory.getFactoryName() + "已与箱码幅面" + qrBoxCodeFormats.get(0).getName() + "绑定，请解绑后再停用。");
            }
            //码包
            QueryWrapper<QrPackage> qrPackageQuery = new QueryWrapper<>();
            qrPackageQuery.eq("factory_id", id);
            qrPackageQuery.eq("is_delete", 0);
            List<QrPackage> qrPackages = qrPackageMapper.selectList(qrPackageQuery);
            if (CollectionUtil.isNotEmpty(qrPackages)) {
                throw new BusinessException("供应商" + factory.getFactoryName() + "已与码包" + qrPackages.get(0).getId() + "绑定，请解绑后再停用。");
            }

            //2-修改状态
            factory.setValid(ValidEnum.NOT_VALID.getCode());
            baseFactoryMapper.updateById(factory);
        }
    }

    /**
     * excel批量导入
     **/
    @Override
    public ExcelImportRes factoryExcelImport(List<FactoryProperty> factoryPropertyList) {
        List<ExcelImportRes.FailBean> failList = new ArrayList<>();

        //1-循环导入，有异常则捕获
        for (int i = 0; i < factoryPropertyList.size(); i++) {
            FactoryProperty factoryProperty = factoryPropertyList.get(i);
            try {
                //1-校验必填与枚举
                String errorMessage = checkFactoryProperty(factoryProperty);
                if (StringUtils.isNotBlank(errorMessage)) {
                    throw new BusinessException(errorMessage);
                }

                //2-调用业务代码新增
                SaveFactoryReq req = new SaveFactoryReq();
                req.setFactoryCode(factoryProperty.getFactoryCode());
                req.setFactoryName(factoryProperty.getFactoryName());
                req.setFactoryType(FactoryTypeEnum.getCode(factoryProperty.getFactoryType()));
                if (FactoryTypeEnum.COVER.is(FactoryTypeEnum.getCode(factoryProperty.getFactoryType()))) {
                    req.setKeymanName(factoryProperty.getKeymanName());
                    req.setKeymanMobile(factoryProperty.getKeymanMobile());
                    req.setEmail(factoryProperty.getEmail());
                }
                req.setMaxFileCount(Long.valueOf(factoryProperty.getMaxFileCount()));
                req.setDescription(factoryProperty.getDescription());

                saveFactory(req);
            } catch (BusinessException e) {
                failList.add(new ExcelImportRes.FailBean(i + 2, e.getMessage()));
            }
        }

        //2-返回导入结果
        ExcelImportRes res = new ExcelImportRes();
        res.setFailedCount(failList.size());
        res.setSuccessCount(factoryPropertyList.size() - res.getFailedCount());
        res.setFailBeanList(failList);

        return res;

    }

    @Override
    public ListFactoryEnableRes listFactoryEnable() {
        QueryWrapper<BaseFactory> param = new QueryWrapper<>();
        param.eq("is_delete", 0);
        param.eq("valid", 1);
        List<BaseFactory> baseDepartments = baseFactoryMapper.selectList(param);
        ListFactoryEnableRes listFactoryEnableRes = new ListFactoryEnableRes();
        List<ListFactoryEnableRes.FactoryBean> factoryBeans = PojoConverterUtils.copyList(baseDepartments, ListFactoryEnableRes.FactoryBean.class);
        listFactoryEnableRes.setFactoryBeanList(factoryBeans);
        return listFactoryEnableRes;
    }

    @Override
    public ListPackageFactorySelectRes listPackageFactorySelect() {
        ListPackageFactorySelectRes listPackageFactorySelectRes = new ListPackageFactorySelectRes();
        QueryWrapper<BaseFactory> param = new QueryWrapper<>();
        //数据权限隔离，如果登录账号是包材厂，只能看账号包材厂的数据。否则看全部
        if (userUtil.getWholeUser().getFactoryId() != null) {
            param.eq("id", userUtil.getWholeUser().getFactoryId());
            listPackageFactorySelectRes.setLoginType(LoginTypeEnum.PACKAGE.getCode());
        } else {
            listPackageFactorySelectRes.setLoginType(LoginTypeEnum.NOT_PACKAGE.getCode());
        }
        param.eq("factory_type", FactoryTypeEnum.COVER.getCode());
        param.eq("is_delete", 0);
        param.eq("valid", 1);
        List<BaseFactory> baseDepartments = baseFactoryMapper.selectList(param);
        List<ListPackageFactorySelectRes.FactoryBean> factoryBeans = PojoConverterUtils.copyList(baseDepartments, ListPackageFactorySelectRes.FactoryBean.class);
        listPackageFactorySelectRes.setFactoryBeanList(factoryBeans);
        return listPackageFactorySelectRes;
    }

    @Override
    public ListBeerFactorySelectRes listBeerFactorySelect() {
        QueryWrapper<BaseFactory> param = new QueryWrapper<>();
        param.eq("factory_type", FactoryTypeEnum.BEER.getCode());
        param.eq("is_delete", 0);
        param.eq("valid", 1);
        List<BaseFactory> baseDepartments = baseFactoryMapper.selectList(param);
        ListBeerFactorySelectRes listBeerFactorySelectRes = new ListBeerFactorySelectRes();
        List<ListBeerFactorySelectRes.FactoryBean> factoryBeans = PojoConverterUtils.copyList(baseDepartments, ListBeerFactorySelectRes.FactoryBean.class);
        listBeerFactorySelectRes.setFactoryBeanList(factoryBeans);
        return listBeerFactorySelectRes;
    }

    @Override
    public ListPageMaterialsRes listPageMaterials() {

        ListPageMaterialsRes res = new ListPageMaterialsRes();
        QueryWrapper<BaseFactory> qw = new QueryWrapper<>();
        qw.eq("factory_type", FactoryTypeEnum.COVER.getCode());
        qw.eq("is_delete", 0);
        qw.eq("valid", 1);

        List<BaseFactory> baseFactoryList = baseFactoryMapper.selectList(qw);
        List<ListPageMaterialsRes.FactoryBean> beanList = PojoConverterUtils.copyList(baseFactoryList, ListPageMaterialsRes.FactoryBean.class);
        res.setFactoryBeanList(beanList);
        return res;
    }

    /**
     * 检查批量导入的内容
     *
     * @param factoryProperty
     * @return
     */
    private String checkFactoryProperty(FactoryProperty factoryProperty) {
        StringBuffer errorMessage = new StringBuffer();
        //逐个字段校验
        if (StringUtils.isBlank(factoryProperty.getFactoryCode())) {
            errorMessage.append("供应商id").append("为空,");
        }
        if (StringUtils.isBlank(factoryProperty.getFactoryName())) {
            errorMessage.append("供应商名称").append("为空,");
        }
        if (StringUtils.isBlank(factoryProperty.getFactoryType())) {
            errorMessage.append("供应商类型").append("为空,");
        }
        if (StringUtils.isNotBlank(factoryProperty.getFactoryType())
                && FactoryTypeEnum.getCode(factoryProperty.getFactoryType()) == null) {
            errorMessage.append("供应商类型").append("枚举值不正确,");
        }
        if (StringUtils.isBlank(factoryProperty.getKeymanName())) {
            errorMessage.append("联系人姓名").append("为空,");
        }
        if (StringUtils.isBlank(factoryProperty.getKeymanMobile())) {
            errorMessage.append("联系人手机号").append("为空,");
        }
        if (StringUtils.isBlank(factoryProperty.getEmail())) {
            errorMessage.append("邮箱").append("为空,");
        }

        return errorMessage.toString();
    }


}
