package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.base.foundation.util.RedisClient;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.dao.entity.UserRole;
import com.abi.qms.platform.dao.mapper.UserInfoMapper;
import com.abi.qms.platform.dao.mapper.UserRoleMapper;
import com.abi.qms.platform.dto.req.LoginReq;
import com.abi.qms.platform.dto.req.ResetPasswordReq;
import com.abi.qms.platform.dto.req.TokenReq;
import com.abi.qms.platform.dto.req.UpdatePasswordReq;
import com.abi.qms.platform.dto.res.LoginRes;
import com.abi.qms.platform.infrastructure.constant.BaseFinal;
import com.abi.qms.platform.infrastructure.constant.RedisKey;
import com.abi.qms.platform.infrastructure.util.RedisUtils;
import com.abi.qms.platform.infrastructure.util.UUIDutils;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.LoginService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.AESEncodeTwoUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.text.MessageFormat;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

import static com.abi.qms.platform.infrastructure.constant.RedisKey.TOKEN_KEY;
import static com.abi.qms.platform.infrastructure.constant.RedisKey.TOKEN_KEY_USER_ID;
import static com.abi.qms.platform.infrastructure.util.JwtTokenUtils.getToken;

/**
 * <p>
 * 登录
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@Service
public class LoginServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements LoginService {

    @Autowired
    private UserInfoMapper userInfoMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private UserUtil userUtil;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public LoginRes login(LoginReq req) {
        UserInfo one = lambdaQuery().eq(UserInfo::getUserName, req.getUserName().trim()).eq(UserInfo::getPassword, req.getPassword()).eq(UserInfo::getIsEnable, 1).eq(UserInfo::getIsDelete, 0).select(UserInfo::getId).one();
        if (Objects.nonNull(one)) {
            UserInfo userInfo = userInfoMapper.selectById(one.getId());
            QueryWrapper<UserRole> roleQueryWrapper = new QueryWrapper<>();
            roleQueryWrapper.eq("role_code", userInfo.getRoleCode());
            roleQueryWrapper.eq("is_delete", 0);
            UserRole userRole = userRoleMapper.selectOne(roleQueryWrapper);
            if (Objects.isNull(userRole)) {
                throw new BusinessException("该用户的角色不存在");
            }
            userUtil.setUser(userInfo);
            String token = getToken(userInfo);
            String redisTokenKey = MessageFormat.format("{0}:{1}",TOKEN_KEY,token);
            redisClient.set(redisTokenKey,true,2, TimeUnit.HOURS);
            redisClient.set(TOKEN_KEY_USER_ID+userInfo.getId(),true,2,TimeUnit.DAYS);
            LoginRes build = LoginRes.builder()
                    .token(MessageFormat.format("Bearer {0}",token))
                    .id(one.getId())
                    .roleCode(one.getRoleCode())
                    .build();
            return build;
        } else {
            throw new BusinessException("登录账号或密码错误");
        }
    }


    @Override
    public void resetPassword(ResetPasswordReq req) {
        UserInfo userInfo = userInfoMapper.selectById(req.getId());
        if (ObjectUtil.isNotEmpty(userInfo)) {
            userInfo.setPassword(req.getPassword());
            userInfoMapper.updateById(userInfo);
            userUtil.releaseUser();
            //删除redis用户token信息
            String userToken = TOKEN_KEY_USER_ID + userInfo.getId();
            if (!redisClient.hasKey(userToken)) {
                return ;
            }
            String token  = redisClient.get(userToken);
            redisClient.delete(TOKEN_KEY_USER_ID + userInfo.getId());
            if (!StringUtils.isEmpty(token)){
                String redisTokenKey = MessageFormat.format("{0}:{1}",TOKEN_KEY,token);
                redisClient.delete(redisTokenKey);
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updatePassword(UpdatePasswordReq req) {
        UserInfo userInfo = userInfoMapper.selectById(req.getId());
        String oldPassword = AESEncodeTwoUtil.encrypt(req.getOldPassword());
        if (oldPassword.equals(userInfo.getPassword())) {
            userInfo.setPassword(req.getNewPassword());
            userInfoMapper.updateById(userInfo);
            String userToken = TOKEN_KEY_USER_ID + userInfo.getId();
            if (!redisClient.hasKey(userToken)) {
                return ;
            }
            String token  = redisClient.get(userToken);
            redisClient.delete(TOKEN_KEY_USER_ID + userInfo.getId());
            if (!StringUtils.isEmpty(token)){
                String redisTokenKey = MessageFormat.format("{0}:{1}",TOKEN_KEY,token);
                redisClient.delete(redisTokenKey);
            }
        } else {
            throw new BusinessException("请输入正确的密码");
        }
    }

    @Override
    public UserInfo validateToken(TokenReq req) {

        return null;
    }
}
