package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.base.foundation.util.JsonUtil;
import com.abi.base.foundation.util.RedisClient;
import com.abi.qms.platform.constant.RedisKeysConstant;
import com.abi.qms.platform.dao.entity.*;
import com.abi.qms.platform.dao.enums.*;
import com.abi.qms.platform.dao.mapper.BaseFactoryMapper;
import com.abi.qms.platform.dao.mapper.QrPackageBatchMapper;
import com.abi.qms.platform.dao.mapper.QrPackageDownloadRecordMapper;
import com.abi.qms.platform.dao.mapper.QrPackageMapper;
import com.abi.qms.platform.dao.vo.result.QrPackageVO;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.CheckQrPackageRepetitiveRes;
import com.abi.qms.platform.dto.res.GetQrPackageDetailRes;
import com.abi.qms.platform.dto.res.ListQrPackageRes;
import com.abi.qms.platform.dto.res.ValidSAPOrderNoAndMaterialRes;
import com.abi.qms.platform.infrastructure.mq.GenerateCodeConsumer;
import com.abi.qms.platform.infrastructure.util.*;
import com.abi.qms.platform.service.QrPackageService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import freemarker.template.Template;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;

import javax.servlet.http.HttpServletResponse;
import java.text.MessageFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 码包 Service业务层处理
 *
 * @author WeiganCai
 * @date 2021-04-28
 */
@Service
@Slf4j
public class QrPackageServiceImpl implements QrPackageService {

    @Autowired
    private QrPackageMapper qrPackageMapper;

    @Autowired
    private BaseFactoryMapper baseFactoryMapper;

    @Autowired
    private QrPackageDownloadRecordMapper qrPackageDownloadRecordMapper;

    @Autowired
    private QrPackageBatchMapper qrPackageBatchMapper;

    @Autowired
    private AmqpTemplate amqpTemplate;

    @Autowired
    private SendmailUtil sendmailUtil;

    @Autowired
    private SmsUtil smsUtil;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private UserUtil userUtil;

    /**
     * 保存码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveQrPackage(SaveQrPackageReq req) {
        boolean isAdd = (req.getId() == null);

        //0-校验入参
        QrPackage qrPackage = checkRequestWhenSaveQrPackage(req);

        //1-自动补全or处理入参数据
        fullParam(req);

        //2-构造码包主表对象
        qrPackage = buildQrPackage(req,qrPackage);

        //3-新增or修改 码包主表
        if (!isAdd) {
            qrPackageMapper.updateById(qrPackage);
        }else{
            qrPackageMapper.insert(qrPackage);
        }

        //4-保存码包批次子表信息
        saveQrPackageBatch(qrPackage,isAdd,req);

    }

    /**
     * 自动补全or处理入参数据
     */
    private void fullParam(SaveQrPackageReq req){
        //1-如果没有传入二级供应商，则表示选的是无等级（即自己做自己）的包材厂，那么就把自己放入二级供应商内
        if(req.getFactoryCoverLevelTwoId()==null){
            req.setFactoryCoverLevelTwoId(req.getFactoryCoverId());
        }

        //2-如果批次信息为空（盖码 or 真实业务没有批次的箱码包材厂），则补充批次信息
        if(CollectionUtil.isEmpty(req.getPackageBatchBeanList())){
            //列表
            List<SaveQrPackageReq.PackageBatchBean> batchBeanList = new ArrayList<>();

            //码的总量
            Long qrNumber = req.getQrNumber();

            //批次信息按包材厂的最大文件数来拆分
            BaseFactory factory = baseFactoryMapper.selectById(req.getFactoryCoverLevelTwoId());
            Long maxFileCount = factory.getMaxFileCount();
            //如果二级包材厂的最大数为空，则认为和总数一致
            if(maxFileCount==null || maxFileCount<=0){
                maxFileCount = qrNumber;
            }

            //计算需要生成多少批次，不是整除则+1
            int batchCount = qrNumber.intValue()/maxFileCount.intValue();
            if(batchCount*maxFileCount<qrNumber){
                batchCount = batchCount+1;
            }

            //循环添加批次
            for(int i=0;i<batchCount;i++){
                //每个批次多少条
                Long batchQrNumber = maxFileCount;
                if(i==batchCount-1){
                    batchQrNumber = qrNumber-(batchCount-1)*maxFileCount;
                }
                SaveQrPackageReq.PackageBatchBean batchBean = new SaveQrPackageReq.PackageBatchBean();
                batchBean.setBatchNumber("000001");
                batchBean.setBatchQrNumber(req.getQrNumber());
                batchBeanList.add(batchBean);
            }
            req.setPackageBatchBeanList(batchBeanList);
        }

        //3-如果是盖码，那么查询盖码得默认箱码幅面，并放入入参
        //TODO


    }

    /**
     * 保存码包批次子表信息
     * @param qrPackage
     * @param isAdd
     * @param req
     */
    private void saveQrPackageBatch(QrPackage qrPackage,boolean isAdd,SaveQrPackageReq req){
        //删除批次号（仅更新码包可能删除）
        if(!isAdd){
            //查询原有批次号
            QueryWrapper<QrPackageBatch> qrPackageBatchQuery = new QueryWrapper<>();
            qrPackageBatchQuery.eq("package_id", qrPackage.getId());
            qrPackageBatchQuery.eq("is_delete", 0);
            List<QrPackageBatch> qrPackageBatches = qrPackageBatchMapper.selectList(qrPackageBatchQuery);
            List<Long> batchOldIds = qrPackageBatches.stream().map(QrPackageBatch::getId).collect(Collectors.toList());
            //更新的批次数据
            List<Long> batchNewIds = req.getPackageBatchBeanList().stream().map(SaveQrPackageReq.PackageBatchBean::getId).collect(Collectors.toList());
            //原有-入参更新 = 删除的数据
            List<Long> deleteBatch = new ArrayList<>(batchOldIds);
            deleteBatch.removeAll(batchNewIds);
            if (CollectionUtil.isNotEmpty(deleteBatch)) {
                UpdateWrapper<QrPackageBatch> qrPackageBatchDeleteParam = new UpdateWrapper<>();
                qrPackageBatchDeleteParam.in("id", deleteBatch)
                        .set("is_delete", 1);
                qrPackageBatchMapper.update(null, qrPackageBatchDeleteParam);
            }
        }

        //添加或更新
        for (SaveQrPackageReq.PackageBatchBean packageBatchBean : req.getPackageBatchBeanList()) {
            if (packageBatchBean.getId() == null) {
                QrPackageBatch qrPackageBatch = new QrPackageBatch();
                qrPackageBatch.setPackageId(qrPackage.getId());
                qrPackageBatch.setBatchNumber(packageBatchBean.getBatchNumber());
                qrPackageBatch.setBatchQrNumber(packageBatchBean.getBatchQrNumber());
                qrPackageBatchMapper.insert(qrPackageBatch);
            } else {
                QueryWrapper<QrPackageBatch> qrPackageBatchQ = new QueryWrapper<>();
                qrPackageBatchQ.eq("id", packageBatchBean.getId());
                qrPackageBatchQ.eq("is_delete", 0);
                QrPackageBatch qrPackageBatch = qrPackageBatchMapper.selectOne(qrPackageBatchQ);
                if (Objects.isNull(qrPackageBatch)) {
                    throw new BusinessException(ErrorCodeEnum.NO_EXISTS_DATA.getCode(), "批次对象不存在");
                }
                qrPackageBatch.setBatchQrNumber(packageBatchBean.getBatchQrNumber());
                qrPackageBatch.setBatchNumber(packageBatchBean.getBatchNumber());
                qrPackageBatchMapper.updateById(qrPackageBatch);
            }
        }
    }


    /**
     * build数据库的码包对象
     * @param req
     * @param qrPackage
     * @return
     */
    private QrPackage buildQrPackage(SaveQrPackageReq req,QrPackage qrPackage){
        if(qrPackage==null){
            qrPackage = new QrPackage();
        }

        qrPackage.setSapOrderNo(req.getSapOrderNo());
        qrPackage.setMaterialId(req.getMaterialId());
        qrPackage.setFactoryCoverId(req.getFactoryCoverId());
        qrPackage.setFactoryCoverLevelTwoId(req.getFactoryCoverLevelTwoId());
        qrPackage.setFactoryBeerId(req.getFactoryBeerId());
        qrPackage.setBoxCodeFormatId(req.getBoxCodeFormatId());
        qrPackage.setQrNumber(req.getQrNumber());
        qrPackage.setStandbyRatio(req.getStandbyRatio());
        qrPackage.setReasonApply(req.getReasonApply());

        return qrPackage;
    }

    /**
     * 校验保存码包的入参
     * @param req
     * @return QrPackage 如果是修改码包就把码包查出来
     */
    private QrPackage checkRequestWhenSaveQrPackage(SaveQrPackageReq req){
        QrPackage qrPackage = null;
        Long qrPackageId = req.getId();
        boolean isAdd = (req.getId() == null);

        // 修改码包时的条件限制
        if (qrPackageId != null) {
            qrPackage = qrPackageMapper.selectById(qrPackageId);
            AssertUtil.isNull(qrPackage, "码包不存在");
            // 只有待审核、审核拒绝状态下才可编辑
            Integer applyStatus = qrPackage.getApplyStatus();
            if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
                throw new BusinessException("码包不可编辑");
            }
        }
        //如果SAP订单号和物料有重复数据，需提交额外申请原因
        ValidSAPOrderNoAndMaterialReq reqValid = new ValidSAPOrderNoAndMaterialReq();
        reqValid.setMaterialId(req.getMaterialId());
        reqValid.setSapOrderNo(req.getSapOrderNo());
        ValidSAPOrderNoAndMaterialRes validSAPOrderNoAndMaterialRes = validSAPOrderNoAndMaterial(reqValid);
        if (Objects.nonNull(validSAPOrderNoAndMaterialRes) && CollectionUtil.isNotEmpty(validSAPOrderNoAndMaterialRes.getPackageBeanList())) {
            Long id = validSAPOrderNoAndMaterialRes.getPackageBeanList().get(0).getId();
            if (validSAPOrderNoAndMaterialRes.getPackageBeanList().size() > 1) {
                if (Objects.isNull(req.getReasonApply())) {
                    throw new BusinessException("该SAP订单号和物料与码包" + id + "重复，若仍需申请需要提交额外码包申请原因");
                }
            }
            if (validSAPOrderNoAndMaterialRes.getPackageBeanList().size() == 1) {
                if (req.getId() == null) {
                    if (Objects.isNull(req.getReasonApply())) {
                        throw new BusinessException("该SAP订单号和物料与码包" + id + "重复，若仍需申请需要提交额外码包申请原因");
                    }
                }
                if (!id.equals(req.getId())) {
                    if (Objects.isNull(req.getReasonApply())) {
                        throw new BusinessException("该SAP订单号和物料与码包" + id + "重复，若仍需申请需要提交额外码包申请原因");
                    }
                }
            }
        }

        //若当前登录用户为包材厂用户，则：
        UserInfo wholeUser = userUtil.getWholeUser();
        BaseFactory factoryCover = baseFactoryMapper.selectById(req.getFactoryCoverId());
        //如果选择的是一级包材厂（而不是无等级包材厂），那么二级包材厂必填
        if(FactoryLevelEnum.ONE.is(factoryCover.getFactoryLevel())
                &&req.getFactoryCoverLevelTwoId()==null){
            throw new BusinessException("选择了一级包材厂，则二级包材厂必填");
        }
        if(wholeUser.getFactoryId() != null&& FactoryTypeEnum.COVER.is(wholeUser.getFactoryType())){
            //1.他不是二级包材厂，包材厂必须为该用户所在的包材厂，不可修改
            if(!FactoryLevelEnum.TWO.is(factoryCover.getFactoryLevel())
                    &&!req.getFactoryCoverId().equals(wholeUser.getFactoryId())){
                throw new BusinessException("包材厂只能选择登录用户的包材厂");
            }
            //2.他是二级包材厂，二级包菜场必须为用户所在的包材厂，不可修改
            if(FactoryLevelEnum.TWO.is(factoryCover.getFactoryLevel())
                    &&!req.getFactoryCoverLevelTwoId().equals(wholeUser.getFactoryId())){
                throw new BusinessException("二级包材厂只能选择登录用户的包材厂");
            }
        }

        if(!isAdd && wholeUser.getFactoryId()!=null){
            //一级包材厂不允许修改码包
            if(FactoryLevelEnum.ONE.is(factoryCover.getFactoryLevel())){
                throw new BusinessException("一级包材厂不允许修改码包");
            }
            //二级包材厂or无级包材厂，只能改自己得码包
            if(!FactoryLevelEnum.ONE.is(factoryCover.getFactoryLevel())
                    && !wholeUser.getFactoryId().equals(qrPackage.getFactoryCoverLevelTwoId())){
                throw new BusinessException("包材厂只允许修改自己的码包");
            }
        }

        //校验批次号
        if (CollectionUtil.isNotEmpty(req.getPackageBatchBeanList())) {
            //同一个码包内，不同批次的批次号不一样
            List<SaveQrPackageReq.PackageBatchBean> packageBatchBeanList = req.getPackageBatchBeanList();
            //聚合批次号
            List<String> batchNumberList = packageBatchBeanList.stream().map(SaveQrPackageReq.PackageBatchBean::getBatchNumber).distinct().collect(Collectors.toList());
            if (batchNumberList.size() < req.getPackageBatchBeanList().size()) {
                throw new BusinessException("批次号重复，请确认后重试");
            }
            //同一个码包内，批次加起来的总数量和码包数量一致
            //计算码包数量总和
            long sum = packageBatchBeanList.stream().mapToLong(SaveQrPackageReq.PackageBatchBean::getBatchQrNumber).sum();
            if (sum != req.getQrNumber()) {
                throw new BusinessException("批次码数量与申请码总量不匹配，请确认后重试");
            }
        }

        return qrPackage;
    }


    /**
     * 码包分页查询
     */
    @Override
    public ListQrPackageRes listQrPackage(ListQrPackageReq req) {
        //数据权限隔离，如果登录账号是包材厂，只能看账号包材厂的数据。否则看全部
        if (userUtil.getWholeUser().getFactoryId() != null) {
            req.setFactoryCoverId(userUtil.getWholeUser().getFactoryId());
        }

        // 分页查询
        IPage<QrPackageVO> iPage = qrPackageMapper.listQrPackage(PageUtil.createPage(req), req);
        List<QrPackageVO> qrPackageList = iPage.getRecords();

        // 封装出参、放入分页信息
        ListQrPackageRes res = new ListQrPackageRes();
        PageUtil.copyPageInfo(res, iPage);
        List<ListQrPackageRes.QrPackageBean> qrPackageBeanList = PojoConverterUtils.copyList(qrPackageList, ListQrPackageRes.QrPackageBean.class);
        res.setQrRepertoryBeanList(qrPackageBeanList);

        return res;
    }

    /**
     * 码包查询详情
     */
    @Override
    public GetQrPackageDetailRes getQrPackageDetail(GetQrPackageDetailReq req) {
        // 查询码包对象
        QrPackageVO qrPackage = qrPackageMapper.selectQrPackageDetailById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");

        return PojoConverterUtils.copy(qrPackage, GetQrPackageDetailRes.class);
    }

    /**
     * 删除码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteQrPackage(DeleteQrPackageReq req) {
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");
        // 只有待审核、审核拒绝状态下才可删除
        Integer applyStatus = qrPackage.getApplyStatus();
        if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
            throw new BusinessException("码包不可删除");
        }

        // 逻辑删除
        QrPackage update = new QrPackage().setId(req.getId()).setIsDelete(1);
        qrPackageMapper.updateById(update);
    }

    /**
     * 审核通过码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void passQrPackage(PassQrPackageReq req) {
        QrPackageVO qrPackage = qrPackageMapper.selectQrPackageDetailById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");

        // 只有待审核、审核拒绝状态下才可通过
        Integer applyStatus = qrPackage.getApplyStatus();
        if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
            throw new BusinessException("码包不可通过");
        }

        QrPackage update = new QrPackage().setId(req.getId()).setApplyStatus(QrPackageApplyStatusEnum.REVIEW_PASS.getCode());
        qrPackageMapper.updateById(update);

        // 码库类型为系统生成时，放入rabbitmq，生成码
        if (QrRepertoryTypeEnum.SYSTEM_GENERATE.is(qrPackage.getQrRepertoryType())) {
            amqpTemplate.convertAndSend(GenerateCodeConsumer.GENERATE_CODE_BY_REPERTORY_QUEUE, JsonUtil.toJson(req.getId()));
        }
    }

    /**
     * 审核拒绝码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void refuseQrPackage(RefuseQrPackageReq req) {
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");

        // 只有待审核、审核拒绝状态下才可拒绝
        Integer applyStatus = qrPackage.getApplyStatus();
        if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
            throw new BusinessException("码包不可拒绝");
        }

        QrPackage update = new QrPackage().setId(req.getId()).setApplyStatus(QrPackageApplyStatusEnum.REVIEW_REFUSE.getCode());
        qrPackageMapper.updateById(update);
    }

    @Override
    public void downloadQrPackage(DownloadQrPackageReq req, HttpServletResponse response) {
        //查询码包
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");
        //查询包材厂
        BaseFactory factory = baseFactoryMapper.selectById(qrPackage.getFactoryCoverId());
        AssertUtil.isNull(factory, "包材厂不存在");
        if (StringUtils.isBlank(factory.getKeymanMobile())) {
            throw new BusinessException("包材厂手机号不存在");
        }

        // 只有已生成状态下才可下载
        Integer generateStatus = qrPackage.getGenerateStatus();
        if (!QrPackageGenerateStatusEnum.GENERATE_SUCCESS.is(generateStatus)) {
            throw new BusinessException("码包不可下载");
        }

        //特殊：校验验证码是否正确，正确才可以下载
        boolean verifyCaptchaResult = verifyCaptcha(factory.getKeymanMobile(), req.getCaptcha(), qrPackage.getId());
        if (!verifyCaptchaResult) {
            throw new BusinessException("验证码错误");
        }

        //开始下载
        String url = qrPackage.getDownloadPath();
        try {
            new OssFileDownloadUtil().downloadFile(url, response);
        } catch (Exception e) {
            log.error("下载失败", e);
            throw new BusinessException("下载失败");
        }

        //插入码包下载记录
        Long userId = userUtil.getUser().getId();
        QrPackageDownloadRecord downloadRecord = new QrPackageDownloadRecord()
                .setCreateBy(userId)
                .setQrPackageId(qrPackage.getId());
        qrPackageDownloadRecordMapper.insert(downloadRecord);

    }

    /**
     * 码生成失败回滚
     */
    @Override
    public void failedRollback(QrPackage qrPackage) {
        // 更改生成状态为生成失败
        QrPackage qrPackageUpdate = new QrPackage().setId(qrPackage.getId()).setGenerateStatus(QrPackageGenerateStatusEnum.GENERATE_FAIL.getCode());
        qrPackageMapper.updateById(qrPackageUpdate);
    }

    /**
     * 校验是否存在SAP订单号和物料重复数据
     * @param req
     * @return
     */
    @Override
    public ValidSAPOrderNoAndMaterialRes validSAPOrderNoAndMaterial(ValidSAPOrderNoAndMaterialReq req) {
        //查询存在的列表
        QueryWrapper<QrPackage> qrPackageQuery = new QueryWrapper<>();
        qrPackageQuery.eq("material_id", req.getMaterialId());
        qrPackageQuery.eq("sap_order_no", req.getSapOrderNo());
        qrPackageQuery.eq("is_delete", 0);
        List<QrPackage> qrPackages = qrPackageMapper.selectList(qrPackageQuery);

        //构造出参
        ValidSAPOrderNoAndMaterialRes res = new ValidSAPOrderNoAndMaterialRes();
        List<ValidSAPOrderNoAndMaterialRes.PackageBean> packageBeanList = PojoConverterUtils.copyList(qrPackages, ValidSAPOrderNoAndMaterialRes.PackageBean.class);
        res.setPackageBeanList(packageBeanList);

        return res;
    }

    /**
     * 码包生成完成后发送邮件给码包厂负责人邮箱
     *
     * @param targetEmail
     * @param sapOrderNo
     * @param filename
     * @param password
     */
    @Override
    public void sendEmailForQrPackageCreated(String targetEmail, String sapOrderNo, String filename, String password) {
        try {
            //拿到freemarker模板
            Template template = FreeMarkerUtils.getTemplate("factory.ftl");

            //填充的内容
            Map paramMap = new HashMap<>();
            paramMap.put("sapOrderNo", sapOrderNo);
            paramMap.put("filename", filename);
            paramMap.put("password", password);

            //生成需要发送的内容
            String emailContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, paramMap);

            sendmailUtil.doSendHtmlEmail("QMS系统通知", emailContent, targetEmail);

        } catch (Exception e) {
            log.info("发送码包通知邮件异常", e);
            throw new BusinessException("发送码包通知邮件异常");
        }
    }

    /**
     * 发送短信验证码
     *
     * @param req
     */
    @Override
    public void sendMobileMsg(SendMobileMsgReq req) {
        //查询码包
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");
        //查询包材厂
        BaseFactory factory = baseFactoryMapper.selectById(qrPackage.getFactoryCoverId());
        AssertUtil.isNull(factory, "包材厂不存在");
        if (StringUtils.isBlank(factory.getKeymanMobile())) {
            throw new BusinessException("包材厂手机号不存在");
        }

        //key:packId:mobile
        Long packId = qrPackage.getId();
        String mobile = factory.getKeymanMobile();
        String key = MessageFormat.format("{0}{1}:{2}", RedisKeysConstant.DOWNLOAD_MOBILE_MSG, packId, mobile);

        //获取随机验证码
        String captcha = RandomCodeUtils.getCaptcha(6);

        //发送短信
        smsUtil.sendCode(mobile, captcha);

        //将验证码存入redis
        redisClient.getRedisTemplate().opsForValue().set(key, captcha, 30 * 60, TimeUnit.SECONDS);
    }

    /**
     * 校验校验码
     *
     * @param mobile
     * @param inputCaptcha
     * @param packId
     * @return
     */
    private boolean verifyCaptcha(String mobile, String inputCaptcha, Long packId) {
        String key = MessageFormat.format("{0}{1}:{2}", RedisKeysConstant.DOWNLOAD_MOBILE_MSG, packId, mobile);
        log.info("验证下载key:{}", key);
        String captcha = redisClient.get(key);
        log.info(inputCaptcha + "--------------------" + captcha);

        //不一致返回false
        if (!inputCaptcha.equals(captcha)) {
            return false;
        }

        //删除校验码
        redisClient.delete(key);

        return true;
    }


}
